#include "tlv_debug.h"
#include "utils/debug.h"

#include <easy/utils.h>


void trace_tlv_map_profile(const struct tlv_map_profile *map_profile)
{
	if (!map_profile)
		return;

	trace("\nTLV type: MAP_TLV_MULTIAP_PROFILE\n");
	trace("\tprofile: %d\n", map_profile->profile);

	trace("\n");
}

void trace_tlv_supported_service(const struct tlv_supported_service *supp_service)
{
	int i;

	if (!supp_service)
		return;

	trace("\nTLV type: MAP_TLV_SUPPORTED_SERVICE\n");
	trace("\tNumber of services: %d\n", supp_service->num_services);

	for (i = 0; i < supp_service->num_services; ++i)
		trace("\tservice[%d]: 0x%X\n", i, supp_service->services[i]);

	trace("\n");
}
#if (EASYMESH_VERSION > 2)
void trace_tlv_akm_suite_caps(const struct tlv_akm_suite_caps *akm_suite_caps)
{
	const struct bbss_akm_suite *bbss_akm_suite =
		(const struct bbss_akm_suite *)akm_suite_caps;
	const struct fbss_akm_suite *fbss_akm_suite;
	const uint8_t *tlv_data = (const uint8_t *)akm_suite_caps;

	int i;

	if (!akm_suite_caps)
		return;

	trace("\nTLV type: MAP_TLV_AKM_SUITE_CAPS\n");
	trace("\tNum Backhaul BSS AKM Suite Selectors: %d\n", bbss_akm_suite->num);

	for (i = 0; i < bbss_akm_suite->num; ++i) {
		const struct akm_suite *suite = bbss_akm_suite->suite + i;

		trace("\t\tbbss_akm_suite[%d]: oui[0]: 0x%X, oui[1]: 0x%X, oui[2]: 0x%X, type: 0x%X\n",
		      i, suite->oui[0], suite->oui[1], suite->oui[2], suite->type);
	}

	fbss_akm_suite =
		(struct fbss_akm_suite *)(tlv_data + sizeof(bbss_akm_suite->num) +
					  (bbss_akm_suite->num * sizeof(bbss_akm_suite->suite[0])));

	trace("\tNum Fronthaul BSS AKM Suite Selectors: %d\n", fbss_akm_suite->num);

	for (i = 0; i < fbss_akm_suite->num; ++i) {
		const struct akm_suite *suite = fbss_akm_suite->suite + i;

		trace("\t\tfbss_akm_suite[%d]: oui[0]: 0x%X, oui[1]: 0x%X, oui[2]: 0x%X, type: 0x%X\n",
		      i, suite->oui[0], suite->oui[1], suite->oui[2], suite->type);
	}

	trace("\n");
}
#endif /* EASYMESH_VERSION > 2 */
void trace_tlv_ap_radio_basic_cap(const struct tlv_ap_radio_basic_cap *ap_radio_basic_cap)
{
	uint8_t *tlv_data = (uint8_t *)ap_radio_basic_cap;
	int offset;
	int i;

	if (!ap_radio_basic_cap)
		return;

	trace("\nTLV type: MAP_TLV_AP_RADIO_BASIC_CAPABILITIES\n");
	trace("\tradio:" MACFMT "\n", MAC2STR(ap_radio_basic_cap->radio));
	trace("\tmax_bssnum: %d\n", ap_radio_basic_cap->max_bssnum);
	trace("\tnum_opclass: %d\n", ap_radio_basic_cap->num_opclass);

	offset = sizeof(*ap_radio_basic_cap);

	for (i = 0; i < ap_radio_basic_cap->num_opclass; ++i) {
		int j;
		const struct ap_radio_basic_cap_opclass *op =
			(struct ap_radio_basic_cap_opclass *)&tlv_data[offset];

		trace("\t\tclassid: %d\n", op->classid);
		trace("\t\tmax_txpower: %d\n", op->max_txpower);
		trace("\t\tnum_nonop_channel: %d\n", op->num_nonop_channel);

		for (j = 0; j < op->num_nonop_channel; ++j)
			trace("\t\t\tnonop_channel: %d\n", op->nonop_channel[j]);

		offset += sizeof(*op) + op->num_nonop_channel;
	}

	trace("\n");
}

void trace_tlv_bsta_radio_cap(const struct tlv_bsta_radio_cap *bsta_radio_cap)
{
	if (!bsta_radio_cap)
		return;

	const bool macaddr_included =
		(bsta_radio_cap->macaddr_included & BSTA_MACADDRESS_INCLUDED);

	trace("\nTLV type: MAP_TLV_BACKHAUL_STA_RADIO_CAPABILITY\n");
	trace("\tradio:" MACFMT "\n", MAC2STR(bsta_radio_cap->radio));
	trace("\tmacaddr_included: %s\n", macaddr_included ? "yes" : "no");

	if (macaddr_included)
		trace("\tmacaddr:" MACFMT "\n", MAC2STR(bsta_radio_cap->macaddr[0]));

	trace("\n");
}

void trace_tlv_profile2_ap_cap(const struct tlv_profile2_ap_cap *profile2_ap_cap)
{
	if (!profile2_ap_cap)
		return;

	trace("\nTLV type: MAP_TLV_PROFILE2_AP_CAPABILITY\n");
	trace("\treserved: 0x%X\n", profile2_ap_cap->reserved);
	trace("\tunit: 0x%X\n", profile2_ap_cap->unit);
	trace("\tmax_vids: %d\n", profile2_ap_cap->max_vids);

	trace("\n");
}

void trace_tlv_ap_radio_adv_cap(const struct tlv_ap_radio_adv_cap *ap_radio_adv_cap)
{
	if (!ap_radio_adv_cap)
		return;

	trace("\nTLV type: MAP_TLV_AP_RADIO_ADV_CAPABILITY\n");
	trace("\tradio:" MACFMT "\n", MAC2STR(ap_radio_adv_cap->radio));
	trace("\tcap: 0x%X\n", ap_radio_adv_cap->cap);

	trace("\n");
}

#if (EASYMESH_VERSION > 2)
void trace_tlv_bss_configuration(const struct tlv *t)
{
	if (!t)
		return;

	if (t->type == MAP_TLV_BSS_CONFIGURATION_REQUEST)
		trace("\nTLV type: MAP_TLV_BSS_CONFIGURATION_REQUEST\n");

	if (t->type == MAP_TLV_BSS_CONFIGURATION_RESPONSE)
		trace("\nTLV type: MAP_TLV_BSS_CONFIGURATION_RESPONSE\n");

	trace("%.*s\n", t->len, (const char *)t->data);

	trace("\n");
}

void trace_tlv_bss_configuration_report(
	const struct tlv_bss_configuration_report *configuration_report)
{
	if (!configuration_report)
		return;

	const uint8_t *tlv_data = (const uint8_t *)configuration_report;
	int i;
	int offset = 0;

	trace("\nTLV type: MAP_TLV_BSS_CONFIGURATION_REPORT\n");

	/* num_radio (1 byte) */
	trace("\nnum_radio: %d\n", configuration_report->num_radio);
	offset += sizeof(configuration_report->num_radio);

	for (i = 0; i < configuration_report->num_radio; ++i) {

		const struct bss_configuration_report_radio *radio =
			(struct bss_configuration_report_radio *)(tlv_data + offset);
		int j;

		/* RUID/macaddr (6 bytes) */
		trace("\truid:" MACFMT "\n", MAC2STR(radio->ruid));
		offset += sizeof(radio->ruid);

		/* num_bss (1 byte) */
		trace("\tnum_bss: %d\n", radio->num_bss);
		offset += sizeof(radio->num_bss);

		for (j = 0; j < radio->num_bss; ++j) {

			const struct bss_configuration_report_bss *config_report =
				(struct bss_configuration_report_bss *)(tlv_data + offset);

			/* BSSID/macaddr (6 bytes) */
			trace("\t\tbssid:" MACFMT "\n", MAC2STR(config_report->bssid));
			offset += sizeof(config_report->bssid);

			/* report flags (1 byte) */
			trace("\t\tflag: 0x%X\n", config_report->flag);
			offset += sizeof(config_report->flag);

			/* reserved flags (1 bytes) */
			trace("\t\trsvd: 0x%X\n", config_report->rsvd);
			offset += sizeof(config_report->rsvd);

			/* ssidlen (1 byte) */
			trace("\t\tssidlen: %d\n", config_report->ssidlen);
			offset += sizeof(config_report->ssidlen);

			/* ssid (ssidlen bytes) */
			trace("\t\tssid: %.*s\n\n", config_report->ssidlen,
			      (const char *)config_report->ssid);
			offset += config_report->ssidlen;
		}
	}

	trace("\n");
}
#endif /* EASYMESH_VERSION > 2 */
